@@ -239,8 +239,43 @@ static void array_add_val(vscf_array_t* a, vscf_data_t* v)
     a->vals[idx] = v;
 }
 
-F_NONNULL F_WUNUSED
-static vscf_simple_t* simple_new(const char* rval, const unsigned rlen)
+/*
+ * Takes a pointer to a constant simple key/value with len
+ * Allocates necessary storage and stores the unescaped version
+ *  in *out, returning the new length, which will be <= the original length
+ * Note also that the returned storage is one byte longer than indicated and
+ *  terminated with a NUL in that extra byte.  It serves two purposes:
+ * (1) Ensuring that the data pointer of a zero-length string/key is not NULL
+ *   (it points to one byte of NUL)
+ * (2) Allowing the treatment of vscf strings as NUL-terminated in cases where
+ *   embedded NULs are irrelevant (such as our own numeric conversions, and
+ *   probably many user-code cases too).
+ */
+F_NONNULLX(2, 3, 4) F_WUNUSED
+static bool unescape_string(vscf_scnr_t* scnr, char** outp, unsigned* outlenp, const char* in, unsigned len)
+{
+    char* out = xmalloc(len + 1);
+    unsigned newlen = len;
+    if (len) {
+        newlen = dns_unescape(out, in, len);
+        gdnsd_assert(newlen <= len);
+        if (!newlen) {
+            if (scnr)
+                parse_error_noargs("Cannot unescape string by DNS escaping rules");
+            free(out);
+            return false;
+        }
+        if (newlen != len)
+            out = xrealloc(out, newlen + 1); // downsize
+    }
+    out[newlen] = 0;
+    *outp = out;
+    *outlenp = newlen;
+    return true;
+}
+
+F_NONNULLX(2) F_WUNUSED
+static vscf_simple_t* simple_new(vscf_scnr_t* scnr, const char* rval, const unsigned rlen)
 {
     vscf_simple_t* s = xcalloc(sizeof(*s));
     char* storage = xmalloc(rlen + 1U);
@@ -249,6 +284,11 @@ static vscf_simple_t* simple_new(const char* rval, const unsigned rlen)
     s->type   = VSCF_SIMPLE_T;
     s->rlen   = rlen;
     s->rval   = storage;
+    if (!unescape_string(scnr, &s->val, &s->len, storage, rlen)) {
+        free(storage);
+        free(s);
+        return NULL;
+    }
     return s;
 }
 
@@ -284,7 +324,15 @@ static vscf_array_t* array_clone(const vscf_array_t* a, const bool ignore_marked
 F_NONNULL F_WUNUSED F_RETNN
 static vscf_simple_t* simple_clone(const vscf_simple_t* s)
 {
-    return simple_new(s->rval, s->rlen);
+    vscf_simple_t* new_s = xcalloc(sizeof(*new_s));
+    new_s->type = VSCF_SIMPLE_T;
+    new_s->rval = xmalloc(s->rlen + 1U);
+    new_s->val = xmalloc(s->len + 1U);
+    new_s->rlen = s->rlen;
+    new_s->len = s->len;
+    memcpy(new_s->rval, s->rval, s->rlen + 1U);
+    memcpy(new_s->val, s->val, s->len + 1U);
+    return new_s;
 }
 
 F_WUNUSED F_RETNN
@@ -307,40 +355,14 @@ static vscf_data_t* val_clone(const vscf_data_t* d, const bool ignore_marked)
     return rv;
 }
 
-/*
- * Takes a pointer to a constant simple key/value with len
- * Allocates necessary storage and stores the unescaped version
- *  in *out, returning the new length, which will be <= the original length
- * Note also that the returned storage is one byte longer than indicated and
- *  terminated with a NUL in that extra byte.  It serves two purposes:
- * (1) Ensuring that the data pointer of a zero-length string/key is not NULL
- *   (it points to one byte of NUL)
- * (2) Allowing the treatment of vscf strings as NUL-terminated in cases where
- *   embedded NULs are irrelevant (such as our own numeric conversions, and
- *   probably many user-code cases too).
- */
-F_NONNULL
-static unsigned unescape_string(char** outp, const char* in, unsigned len)
-{
-    char* out = xmalloc(len + 1);
-    unsigned newlen = len;
-    if (len) {
-        newlen = dns_unescape(out, in, len);
-        gdnsd_assert(newlen && newlen <= len);
-        if (newlen != len)
-            out = xrealloc(out, newlen + 1); // downsize
-    }
-    out[newlen] = 0;
-    *outp = out;
-    return newlen;
-}
-
-F_NONNULL
-static void set_key(vscf_scnr_t* scnr, const char* end)
+F_NONNULL F_WUNUSED
+static bool set_key(vscf_scnr_t* scnr, const char* end)
 {
     gdnsd_assert(scnr->tstart);
-    scnr->cur_klen = unescape_string(&scnr->cur_key, scnr->tstart, end - scnr->tstart);
+    if (!unescape_string(scnr, &scnr->cur_key, &scnr->cur_klen, scnr->tstart, end - scnr->tstart))
+        return false;
     scnr->tstart = NULL;
+    return true;
 }
 
 F_NONNULL F_WUNUSED
@@ -365,7 +387,9 @@ static bool scnr_set_simple(vscf_scnr_t* scnr, const char* end)
 {
     gdnsd_assert(scnr->tstart);
     const unsigned rlen = end - scnr->tstart;
-    vscf_simple_t* s = simple_new(scnr->tstart, rlen);
+    vscf_simple_t* s = simple_new(scnr, scnr->tstart, rlen);
+    if (!s)
+        return false;
     scnr->tstart = NULL;
     return add_to_cur_container(scnr, (vscf_data_t*)s);
 }
@@ -373,7 +397,7 @@ static bool scnr_set_simple(vscf_scnr_t* scnr, const char* end)
 static void val_destroy(vscf_data_t* d);
 
 F_NONNULL F_WUNUSED
-static bool vscf_include_file(vscf_scnr_t* scnr, const char* fn, const size_t idx)
+static bool vscf_include_file(vscf_scnr_t* scnr, const char* fn)
 {
     vscf_data_t* inc_data = vscf_scan_filename(fn);
 
@@ -402,11 +426,6 @@ static bool vscf_include_file(vscf_scnr_t* scnr, const char* fn, const size_t id
         }
         val_destroy(inc_data);
     } else { // value context
-        if (idx > 1) {
-            parse_error("Include file '%s': cannot include multiple files in value context", fn);
-            val_destroy(inc_data);
-            return false;
-        }
         return add_to_cur_container(scnr, inc_data);
     }
 
@@ -426,8 +445,17 @@ static bool vscf_include_glob(vscf_scnr_t* scnr, const char* inc_glob, const int
     }
 
     if (globrv != GLOB_NOMATCH) {
+        vscf_data_t* cont = scnr->cont_stack[scnr->cont_stack_top];
+        if (!vscf_is_hash(cont) || scnr->cur_key) { // value-context
+            if (globbuf.gl_pathc > 1U) {
+                parse_error("Including '%s': cannot include multiple files in value context", inc_glob);
+                globfree(&globbuf);
+                return false;
+            }
+        }
+
         for (size_t i = 0; i < globbuf.gl_pathc; i++) {
-            if (!vscf_include_file(scnr, globbuf.gl_pathv[i], i)) {
+            if (!vscf_include_file(scnr, globbuf.gl_pathv[i])) {
                 globfree(&globbuf);
                 return false;
             }
@@ -511,13 +539,6 @@ static bool scnr_proc_include(vscf_scnr_t* scnr, const char* end)
     return rv;
 }
 
-F_NONNULL
-static void vscf_simple_ensure_val(vscf_simple_t* s)
-{
-    if (!s->val)
-        s->len = unescape_string(&s->val, s->rval, s->rlen);
-}
-
 F_NONNULL F_WUNUSED
 static bool cont_stack_push(vscf_scnr_t* scnr, vscf_data_t* c)
 {
@@ -598,7 +619,7 @@ static void val_destroy(vscf_data_t* d)
 // *INDENT-OFF*
 // start-sonar-exclude
 
-#line 602 "libgdnsd/vscf.c"
+#line 623 "libgdnsd/vscf.c"
 static const char _vscf_actions[] = {
 	0, 1, 0, 1, 1, 1, 2, 1, 
 	3, 1, 4, 1, 5, 1, 6, 1, 
@@ -1252,7 +1273,7 @@ static const int vscf_en_hash = 140;
 static const int vscf_en_main = 200;
 
 
-#line 742 "./libgdnsd/vscf.rl"
+#line 767 "./libgdnsd/vscf.rl"
 
 // end-sonar-exclude
 // *INDENT-ON*
@@ -1284,13 +1305,13 @@ vscf_data_t* vscf_scan_buf(const size_t len, const char* buf, const char* source
     // *INDENT-OFF*
     // start-sonar-exclude
     
-#line 783 "./libgdnsd/vscf.rl"
+#line 808 "./libgdnsd/vscf.rl"
 
 
 GDNSD_DIAG_PUSH_IGNORED("-Wswitch-default")
 GDNSD_DIAG_PUSH_IGNORED("-Wsign-conversion")
     
-#line 1294 "libgdnsd/vscf.c"
+#line 1315 "libgdnsd/vscf.c"
 	{
 	int _klen;
 	unsigned int _trans;
@@ -1364,29 +1385,33 @@ _match:
 		switch ( *_acts++ )
 		{
 	case 0:
-#line 605 "./libgdnsd/vscf.rl"
+#line 626 "./libgdnsd/vscf.rl"
 	{ scnr->tstart = (     scnr->p); }
 	break;
 	case 1:
-#line 607 "./libgdnsd/vscf.rl"
-	{ set_key(scnr, (     scnr->p)); }
+#line 628 "./libgdnsd/vscf.rl"
+	{
+        if (!set_key(scnr, (     scnr->p)))
+            {(     scnr->p)++; goto _out; }
+    }
 	break;
 	case 2:
-#line 609 "./libgdnsd/vscf.rl"
+#line 633 "./libgdnsd/vscf.rl"
 	{
         scnr->tstart++;
-        set_key(scnr, (     scnr->p) - 1);
+        if (!set_key(scnr, (     scnr->p) - 1))
+            {(     scnr->p)++; goto _out; }
     }
 	break;
 	case 3:
-#line 614 "./libgdnsd/vscf.rl"
+#line 639 "./libgdnsd/vscf.rl"
 	{
         if (!scnr_set_simple(scnr, (     scnr->p)))
             {(     scnr->p)++; goto _out; }
     }
 	break;
 	case 4:
-#line 619 "./libgdnsd/vscf.rl"
+#line 644 "./libgdnsd/vscf.rl"
 	{
         scnr->tstart++;
         if (!scnr_set_simple(scnr, (     scnr->p) - 1))
@@ -1394,11 +1419,11 @@ _match:
     }
 	break;
 	case 5:
-#line 626 "./libgdnsd/vscf.rl"
+#line 651 "./libgdnsd/vscf.rl"
 	{ scnr->lcount++; }
 	break;
 	case 6:
-#line 664 "./libgdnsd/vscf.rl"
+#line 689 "./libgdnsd/vscf.rl"
 	{
         if (!cont_stack_push(scnr, (vscf_data_t*)array_new()))
             {(     scnr->p)++; goto _out; }
@@ -1409,7 +1434,7 @@ _match:
     }
 	break;
 	case 7:
-#line 670 "./libgdnsd/vscf.rl"
+#line 695 "./libgdnsd/vscf.rl"
 	{
         if (!cont_stack_push(scnr, (vscf_data_t*)hash_new()))
             {(     scnr->p)++; goto _out; }
@@ -1420,21 +1445,21 @@ _match:
     }
 	break;
 	case 8:
-#line 676 "./libgdnsd/vscf.rl"
+#line 701 "./libgdnsd/vscf.rl"
 	{
         cont_stack_pop(scnr);
         {(    scnr->cs) = ( scnr->cs_stack)[--(   scnr->top)]; goto _again;}
     }
 	break;
 	case 9:
-#line 681 "./libgdnsd/vscf.rl"
+#line 706 "./libgdnsd/vscf.rl"
 	{
         cont_stack_pop(scnr);
         {(    scnr->cs) = ( scnr->cs_stack)[--(   scnr->top)]; goto _again;}
     }
 	break;
 	case 10:
-#line 686 "./libgdnsd/vscf.rl"
+#line 711 "./libgdnsd/vscf.rl"
 	{
         gdnsd_assert(scnr->cont_stack_top == 0); // outermost
         gdnsd_assert(vscf_is_hash(scnr->cont_stack[0])); // default hash
@@ -1443,21 +1468,21 @@ _match:
     }
 	break;
 	case 11:
-#line 693 "./libgdnsd/vscf.rl"
+#line 718 "./libgdnsd/vscf.rl"
 	{
         if (!scnr_proc_include(scnr, (     scnr->p)))
             {(     scnr->p)++; goto _out; }
     }
 	break;
 	case 12:
-#line 698 "./libgdnsd/vscf.rl"
+#line 723 "./libgdnsd/vscf.rl"
 	{
         scnr->tstart++;
         if (!scnr_proc_include(scnr, (     scnr->p) - 1))
             {(     scnr->p)++; goto _out; }
     }
 	break;
-#line 1461 "libgdnsd/vscf.c"
+#line 1486 "libgdnsd/vscf.c"
 		}
 	}
 
@@ -1474,14 +1499,14 @@ _again:
 	while ( __nacts-- > 0 ) {
 		switch ( *__acts++ ) {
 	case 3:
-#line 614 "./libgdnsd/vscf.rl"
+#line 639 "./libgdnsd/vscf.rl"
 	{
         if (!scnr_set_simple(scnr, (     scnr->p)))
             {(     scnr->p)++; goto _out; }
     }
 	break;
 	case 4:
-#line 619 "./libgdnsd/vscf.rl"
+#line 644 "./libgdnsd/vscf.rl"
 	{
         scnr->tstart++;
         if (!scnr_set_simple(scnr, (     scnr->p) - 1))
@@ -1489,10 +1514,10 @@ _again:
     }
 	break;
 	case 5:
-#line 626 "./libgdnsd/vscf.rl"
+#line 651 "./libgdnsd/vscf.rl"
 	{ scnr->lcount++; }
 	break;
-#line 1496 "libgdnsd/vscf.c"
+#line 1521 "libgdnsd/vscf.c"
 		}
 	}
 	}
@@ -1500,7 +1525,7 @@ _again:
 	_out: {}
 	}
 
-#line 788 "./libgdnsd/vscf.rl"
+#line 813 "./libgdnsd/vscf.rl"
 GDNSD_DIAG_POP
 GDNSD_DIAG_POP
     // end-sonar-exclude
@@ -1593,14 +1618,12 @@ vscf_data_t* vscf_get_parent(const vscf_data_t* d)
 unsigned vscf_simple_get_len(vscf_data_t* d)
 {
     gdnsd_assert(vscf_is_simple(d));
-    vscf_simple_ensure_val(&d->simple);
     return d->simple.len;
 }
 
 const char* vscf_simple_get_data(vscf_data_t* d)
 {
     gdnsd_assert(vscf_is_simple(d));
-    vscf_simple_ensure_val(&d->simple);
     return d->simple.val;
 }
 
@@ -1723,7 +1746,6 @@ void vscf_hash_sort(const vscf_data_t* d, vscf_key_cmp_cb_t f)
 bool vscf_simple_get_as_ulong(vscf_data_t* d, unsigned long* out)
 {
     gdnsd_assert(vscf_is_simple(d));
-    vscf_simple_ensure_val(&d->simple);
     if (!d->simple.len)
         return false;
     char* eptr;
@@ -1742,7 +1764,6 @@ bool vscf_simple_get_as_ulong(vscf_data_t* d, unsigned long* out)
 bool vscf_simple_get_as_long(vscf_data_t* d, long* out)
 {
     gdnsd_assert(vscf_is_simple(d));
-    vscf_simple_ensure_val(&d->simple);
     if (!d->simple.len)
         return false;
     char* eptr;
@@ -1761,7 +1782,6 @@ bool vscf_simple_get_as_long(vscf_data_t* d, long* out)
 bool vscf_simple_get_as_double(vscf_data_t* d, double* out)
 {
     gdnsd_assert(vscf_is_simple(d));
-    vscf_simple_ensure_val(&d->simple);
     if (!d->simple.len)
         return false;
     char* eptr;
@@ -1780,7 +1800,6 @@ bool vscf_simple_get_as_double(vscf_data_t* d, double* out)
 bool vscf_simple_get_as_bool(vscf_data_t* d, bool* out)
 {
     gdnsd_assert(vscf_is_simple(d));
-    vscf_simple_ensure_val(&d->simple);
     if (d->simple.len == 4
             && (d->simple.val[0] == 'T' || d->simple.val[0] == 't')
             && (d->simple.val[1] == 'R' || d->simple.val[1] == 'r')
@@ -1821,7 +1840,7 @@ vscf_data_t* vscf_array_new(void)
 
 vscf_data_t* vscf_simple_new(const char* rval, const unsigned rlen)
 {
-    return (vscf_data_t*)simple_new(rval, rlen);
+    return (vscf_data_t*)simple_new(NULL, rval, rlen);
 }
 
 void vscf_array_add_val(vscf_data_t* a, vscf_data_t* v)
