@@ -246,18 +246,16 @@ static void process_listen(socks_cfg_t* socks_cfg, vscf_data_t* listen_opt, cons
         dns_addr_t* a = &socks_cfg->dns_addrs[i];
 
         for (unsigned j = 0; j < a->udp_threads; j++) {
-            dns_thread_t* t = &socks_cfg->dns_threads[tnum];
+            dns_thread_t* t = &socks_cfg->dns_threads[tnum++];
             t->ac = a;
             t->is_udp = true;
-            t->threadnum = tnum++;
             t->sock = -1;
         }
 
         for (unsigned j = 0; j < a->tcp_threads; j++) {
-            dns_thread_t* t = &socks_cfg->dns_threads[tnum];
+            dns_thread_t* t = &socks_cfg->dns_threads[tnum++];
             t->ac = a;
             t->is_udp = false;
-            t->threadnum = tnum++;
             t->sock = -1;
         }
 
